import { Injectable, NgModule, Component, Input, defineInjectable } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AngularDataTableService {
    constructor() { }
    /**
     * @param {?} totalItems
     * @param {?=} currentPage
     * @param {?=} pageSize
     * @return {?}
     */
    getPager(totalItems, currentPage = 1, pageSize = 12) {
        // calculate total pages
        /** @type {?} */
        const totalPages = Math.ceil(totalItems / pageSize);
        // ensure current page isn't out of range
        if (currentPage < 1) {
            currentPage = 1;
        }
        else if (currentPage > totalPages) {
            currentPage = totalPages;
        }
        /** @type {?} */
        let startPage;
        /** @type {?} */
        let endPage;
        if (totalPages <= 12) {
            // less than 10 total pages so show all
            startPage = 1;
            endPage = totalPages;
        }
        else {
            // more than 10 total pages so calculate start and end pages
            if (currentPage <= 6) {
                startPage = 1;
                endPage = 12;
            }
            else if (currentPage + 4 >= totalPages) {
                startPage = totalPages - 9;
                endPage = totalPages;
            }
            else {
                startPage = currentPage - 5;
                endPage = currentPage + 4;
            }
        }
        // calculate start and end item indexes
        /** @type {?} */
        const startIndex = (currentPage - 1) * pageSize;
        /** @type {?} */
        const endIndex = Math.min(startIndex + pageSize - 1, totalItems - 1);
        // create an array of pages to ng-repeat in the pager control
        /** @type {?} */
        const pages = Array.from(Array((endPage + 1) - startPage).keys()).map((/**
         * @param {?} i
         * @return {?}
         */
        i => startPage + i));
        // return object with all pager properties required by the view
        return {
            totalItems: totalItems,
            currentPage: currentPage,
            pageSize: pageSize,
            totalPages: totalPages,
            startPage: startPage,
            endPage: endPage,
            startIndex: startIndex,
            endIndex: endIndex,
            pages: pages
        };
    }
}
AngularDataTableService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
AngularDataTableService.ctorParameters = () => [];
/** @nocollapse */ AngularDataTableService.ngInjectableDef = defineInjectable({ factory: function AngularDataTableService_Factory() { return new AngularDataTableService(); }, token: AngularDataTableService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AngularDataTableComponent$$1 {
    /**
     * @param {?} angularDataTableService
     */
    constructor(angularDataTableService) {
        this.angularDataTableService = angularDataTableService;
        // pager object
        this.pager = {};
        this.index = 1;
        this.bgColor = 'Black';
        this.currentPage = 1;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        setTimeout((/**
         * @return {?}
         */
        () => {
            if (this.data && this.data.length > 0) {
                this.totalItems = this.data.length;
                this.setPage(1);
            }
        }));
    }
    /**
     * @param {?} fieldName
     * @param {?} sortOrder
     * @return {?}
     */
    Sorting(fieldName, sortOrder) {
        if (sortOrder) {
            this.data.sort((/**
             * @param {?} objectOne
             * @param {?} objectTwo
             * @return {?}
             */
            function (objectOne, objectTwo) {
                if (objectOne[fieldName] < objectTwo[fieldName]) {
                    return -1;
                }
                if (objectOne[fieldName] > objectTwo[fieldName]) {
                    return 1;
                }
                return 0;
            }));
        }
        else {
            this.data.sort((/**
             * @param {?} objectOne
             * @param {?} objectTwo
             * @return {?}
             */
            function (objectOne, objectTwo) {
                if (objectOne[fieldName] > objectTwo[fieldName]) {
                    return -1;
                }
                if (objectOne[fieldName] < objectTwo[fieldName]) {
                    return 1;
                }
                return 0;
            }));
        }
        this.setPage(this.currentPage);
    }
    /**
     * @param {?} page
     * @return {?}
     */
    setPage(page) {
        if (this.color) {
            this.bgColor = this.color;
        }
        this.index = 1;
        this.data.forEach((/**
         * @param {?} element
         * @return {?}
         */
        (element) => {
            element._index = this.index++;
        }));
        // get pager object from service
        this.pager = this.angularDataTableService.getPager(this.data.length, page, this.itemPerPage);
        // get current page of items
        this.pagedItems = this.data.slice(this.pager.startIndex, this.pager.endIndex + 1);
        // get object property name
        this.propName = Object.getOwnPropertyNames(this.data[0]);
        this.propName.pop();
    }
    /**
     * @return {?}
     */
    nextPage() {
        if (this.currentPage < this.pager.pages.length) {
            this.currentPage = this.currentPage + 1;
            this.setPage(this.currentPage);
        }
    }
    /**
     * @return {?}
     */
    previousPage() {
        if (this.currentPage > 1) {
            this.currentPage = this.currentPage - 1;
            this.setPage(this.currentPage);
        }
    }
}
AngularDataTableComponent$$1.decorators = [
    { type: Component, args: [{
                // tslint:disable-next-line:component-selector
                selector: 'ang-data-table',
                template: `<!-- <div class="header">
  <div class="two" *ngIf="pagedItems">Show
    <select>
      <option value="10">10</option>
      <option value="15">15</option>
      <option value="58">25</option>
      <option value="100">100</option>
    </select> entries
  </div>

  <div class="one" *ngIf="pager.pages && pager.pages.length > 1">
    Search: <input type="text">
  </div>
</div> -->

<div *ngIf="pagedItems">

<div style="overflow-x:auto;">
  <table>
    <thead>
      <th *ngFor="let prop of propName | keyvalue" [style.background-color]="bgColor">
        {{ prop.value | titlecase  }} <i class="fa fa-sort" (click)="Sorting(prop.value, !sortOrder)" (click)="sortOrder = !sortOrder">
        </i></th>
    </thead>
    <tbody>
      <tr *ngFor="let item of pagedItems ">
        <td *ngFor="let prop of propName"> {{ item[prop] }} </td>
      </tr>
    </tbody>
  </table>
</div>

<br />
<div class="header" style="overflow-x:auto;">
  <div class="two" *ngIf="pagedItems">Showing {{pagedItems[0]._index}} to
    {{pagedItems[pagedItems.length-1]._index}} of
    {{totalItems}} entries</div>

  <div class="one" *ngIf="pager.pages && pager.pages.length > 1">
    <button class="button button2" [style.background-color]="bgColor" (click)="previousPage()">Previous</button>
    <button class="button button2" [style.background-color]="bgColor" (click)="nextPage()">Next</button>
  </div>
</div>

<div class="center">
  Page {{pager.currentPage}}
</div>

</div>

<!-- CSS -->
<link href="//maxcdn.bootstrapcdn.com/font-awesome/4.1.0/css/font-awesome.min.css" rel="stylesheet">
<style>
  table,
  td,
  th {
    border: 1px solid #ddd;
    text-align: left;
  }

  table {
    border-collapse: collapse;
    width: 100%;
  }

  th,
  td {
    padding: 15px;
  }

  th {
    /* background-color: #24292e; */
    color: white;
  }

  tr:hover {
    background-color: #f5f5f5;
  }

  .button {
    /* background-color: #24292e; */
    border: none;
    color: white;
    padding: 10px 24px;
    text-align: center;
    text-decoration: none;
    display: inline-block;
    font-size: 16px;
    margin: 4px 2px;
    cursor: pointer;
    -webkit-transition-duration: 0.4s;
    /* Safari */
    transition-duration: 0.4s;
  }

  .button2:hover {
    box-shadow: 0 12px 16px 0 rgba(0, 0, 0, 0.24), 0 17px 50px 0 rgba(0, 0, 0, 0.19);
  }

  .center {
    text-align: center;
  }


  .header {
    display: inline-block;
    width: 100%;
  }

  .one {
    float: right;
  }

  .two {
    float: left;
  }
</style>
<!-- CSS --> `
            }] }
];
/** @nocollapse */
AngularDataTableComponent$$1.ctorParameters = () => [
    { type: AngularDataTableService }
];
AngularDataTableComponent$$1.propDecorators = {
    data: [{ type: Input }],
    itemPerPage: [{ type: Input }],
    color: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AngularDataTableModule {
}
AngularDataTableModule.decorators = [
    { type: NgModule, args: [{
                declarations: [AngularDataTableComponent$$1],
                imports: [
                    BrowserModule
                ],
                exports: [AngularDataTableComponent$$1]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AngularDataTableService, AngularDataTableComponent$$1 as AngularDataTableComponent, AngularDataTableModule };

//# sourceMappingURL=angular-data-table-library.js.map